<?php

class VBSEOImporter_Importer_vBSEO extends XenForo_Importer_Abstract
{

	/**
	 * Source database connection.
	 *
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_sourceDb;
	
	protected $_prefix;
	
	protected $_charset = 'windows-1252';
	
	protected $_config;
	
	public static function getName()
	{
		return 'vBSEO Likes';
	}
	
	public function configure(XenForo_ControllerAdmin_Abstract $controller, array &$config)
	{
		if ($config)
		{
			if ($errors = $this->validateConfiguration($config))
			{
				return $controller->responseError($errors);
			}
	
			$this->_bootstrap($config);
	
			return true;
		}
	
		$configPath = getcwd() . '/includes/config.php';
		if (file_exists($configPath) && is_readable($configPath))
		{
			$config = array();
			include($configPath);
	
			$viewParams = array('input' => $config);
		}
		else
		{
			$viewParams = array('input' => array
					(
							'MasterServer' => array
							(
									'servername' => 'localhost',
									'port' => 3306,
									'username' => '',
									'password' => '',
							),
							
							'Database' => array
							(
									'dbname' => '',
									'tableprefix' => ''
							),
							
							'Mysqli' => array
							(
									'charset' => ''
							),
					));
		}
	
		return $controller->responseView('XenForo_ViewAdmin_Import_vBulletin_Config', 'vbseoimporter_import_config', $viewParams);
	}
	
	public function validateConfiguration(array &$config)
	{
		$errors = array();
	
		if (preg_match('/[^a-z0-9_]/i', $config['importlog']))
		{
			$errors[] = new XenForo_Phrase('error_table_name_illegal');
		}
		else
		{
			try
			{
				$table = $this->_db->describeTable($config['importlog']);
				if (!isset($table['content_type']) || !isset($table['old_id']) || !isset($table['new_id']))
					$errors[] = new XenForo_Phrase('vbseoimporter_import_log_table_invalid');
			}
			catch (Zend_Db_Exception $e)
			{
				$errors[] = $e->getMessage();
			}
		}
	
		$config['db']['prefix'] = preg_replace('/[^a-z0-9_]/i', '', $config['db']['prefix']);
	
		try
		{
			$db = Zend_Db::factory('mysqli',
					array(
							'host' => $config['db']['host'],
							'port' => $config['db']['port'],
							'username' => $config['db']['username'],
							'password' => $config['db']['password'],
							'dbname' => $config['db']['dbname'],
							'charset' => $config['db']['charset']
					)
			);
			$db->getConnection();
		}
		catch (Zend_Db_Exception $e)
		{
			$errors[] = new XenForo_Phrase('source_database_connection_details_not_correct_x', array('error' => $e->getMessage()));
		}
	
		if ($errors)
		{
			return $errors;
		}
	
		try
		{
			$db->query('
				SELECT userid
				FROM ' . $config['db']['prefix'] . 'user
				LIMIT 1
			');
		}
		catch (Zend_Db_Exception $e)
		{
			if ($config['db']['dbname'] === '')
			{
				$errors[] = new XenForo_Phrase('please_enter_database_name');
			}
			else
			{
				$errors[] = new XenForo_Phrase('table_prefix_or_database_name_is_not_correct');
			}
		}
	
		if (!$errors)
		{
			$defaultLanguageId = $db->fetchOne('
				SELECT value
				FROM ' . $config['db']['prefix'] . 'setting
				WHERE varname = \'languageid\'
			');
			$defaultCharset = $db->fetchOne('
				SELECT charset
				FROM ' . $config['db']['prefix'] . 'language
				WHERE languageid = ?
			', $defaultLanguageId);
			if (!$defaultCharset || str_replace('-', '', strtolower($defaultCharset)) == 'iso88591')
			{
				$config['charset'] = 'windows-1252';
			}
			else
			{
				$config['charset'] = strtolower($defaultCharset);
			}
	
			$config['retain_keys'] = false;
		}
	
		return $errors;
	}
	
	public function getSteps()
	{
		return array(
				'vBSEOLikes' => array(
						'title' => 'Import vBSEO Likes'
				),
		);
	}
	
	public function stepvBSEOLikes($start, array $options)
	{
		
		define('IMPORT_LOG_TABLE', $this->_config['importlog']);
		
		$options = array_merge(array(
				'fetchLikeUsers' => false,
				'limit' => 200,
				'max' => false
		), $options);
		
		$sDb = $this->_sourceDb;
		$prefix = $this->_prefix;
		
		$model = $this->_importModel;
		
		if ($options['max'] === false)
		{
			$options['max'] = $sDb->fetchOne('
				SELECT MAX(l_contentid)
				FROM ' . $prefix . 'vbseo_likes
			');
		}
		
		$likes = $sDb->fetchAll($sDb->limit(
				'
				SELECT *
				FROM ' . $prefix . 'vbseo_likes
				WHERE l_contentid > ' . $sDb->quote($start) . '
				ORDER BY l_contentid
			', $options['limit']
		));
		
		if (!$likes)
		{
			return true;
		}
		
		$next = 0;
		$total = 0;

		$userIds = array();
		foreach ($likes AS $like)
		{
			$userIds[] = $like['l_from_userid'];
			$userIds[] = $like['l_dest_userid'];
		}

		$postIdMap = $model->getPostIdsMapFromArray($likes, 'l_contentid');
		$userIdMap = $model->getImportContentMap('user', $userIds);

		XenForo_Db::beginTransaction();

		foreach ($likes AS $like)
		{
			$next = $like['l_contentid'];

			$newPostId = $this->_mapLookUp($postIdMap, $like['l_contentid']);
			if (!$newPostId)
			{
				continue;
			}

			$model->importLike(
				'post',
				$newPostId,
				$this->_mapLookUp($userIdMap, $like['l_dest_userid']),
				$this->_mapLookUp($userIdMap, $like['l_from_userid']),
				$like['l_dateline'],
				$options['fetchLikeUsers']
			);

			$total++;
		}

		XenForo_Db::commit();

		$this->_session->incrementStepImportTotal($total);

		return array($next, $options, $this->_getProgressOutput($next, $options['max']));
	}
	
	protected function _bootstrap(array $config)
	{
		if ($this->_sourceDb)
		{
			// already run
			return;
		}
	
		@set_time_limit(0);
	
		$this->_config = $config;
	
		$this->_sourceDb = Zend_Db::factory('mysqli',
				array(
						'host' => $config['db']['host'],
						'port' => $config['db']['port'],
						'username' => $config['db']['username'],
						'password' => $config['db']['password'],
						'dbname' => $config['db']['dbname'],
						'charset' => $config['db']['charset']
				)
		);
	
		$this->_prefix = preg_replace('/[^a-z0-9_]/i', '', $config['db']['prefix']);
	
		if (!empty($config['charset']))
		{
			$this->_charset = $config['charset'];
		}
	}
	
	public function __destruct()
	{
		if ($this->_sourceDb instanceof Zend_Db_Adapter_Abstract)
		{
			$this->_sourceDb->closeConnection();
		}
	}

}